﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Windows.Forms;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;
using HeuristicLab.PluginInfrastructure;
using HeuristicLab.Optimization;
using HeuristicLab.Optimization.Views;

namespace HeuristicLab.Hive.Experiment.Views {
  /// <summary>
  /// The base class for visual representations of items.
  /// </summary>
  [View("Experiment View")]
  [Content(typeof(HiveExperiment), true)]
  public sealed partial class HiveExperimentView : NamedItemView {
    public new HiveExperiment Content {
      get { return (HiveExperiment)base.Content; }
      set { base.Content = value; }
    }

    /// <summary>
    /// Initializes a new instance of <see cref="ItemBaseView"/>.
    /// </summary>
    public HiveExperimentView() {
      InitializeComponent();
    }

    protected override void DeregisterContentEvents() {
      Content.ExceptionOccurred -= new EventHandler<EventArgs<Exception>>(Content_ExceptionOccurred);
      Content.ExecutionStateChanged -= new EventHandler(Content_ExecutionStateChanged);
      Content.ExecutionTimeChanged -= new EventHandler(Content_ExecutionTimeChanged);
      Content.Prepared -= new EventHandler(Content_Prepared);
      Content.Started -= new EventHandler(Content_Started);
      Content.Paused -= new EventHandler(Content_Paused);
      Content.Stopped -= new EventHandler(Content_Stopped);
      Content.ExperimentChanged -= new EventHandler(Content_ExperimentChanged);
      base.DeregisterContentEvents();
    }
    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      Content.ExceptionOccurred += new EventHandler<EventArgs<Exception>>(Content_ExceptionOccurred);
      Content.ExecutionStateChanged += new EventHandler(Content_ExecutionStateChanged);
      Content.ExecutionTimeChanged += new EventHandler(Content_ExecutionTimeChanged);
      Content.Prepared += new EventHandler(Content_Prepared);
      Content.Started += new EventHandler(Content_Started);
      Content.Paused += new EventHandler(Content_Paused);
      Content.Stopped += new EventHandler(Content_Stopped);
      Content.ExperimentChanged += new EventHandler(Content_ExperimentChanged);
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content == null) {
        executionTimeTextBox.Text = string.Empty;
      } else {
        executionTimeTextBox.Text = Content.ExecutionTime.ToString();
        resourceIdsTextBox.Text = Content.ResourceIds;
        serverUrlTextBox.Text = Content.ServerUrl;
        experimentNamedItemView.Content = Content.Experiment;
        logView.Content = Content.Log;
        jobListView.Content = Content.JobItems;
      }
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      executionTimeTextBox.Enabled = Content != null;
      if (Content != null) {
        openExperimentButton.Enabled = Content.Experiment != null;
        experimentNamedItemView.Locked = Content.Experiment == null;
      }
      SetEnabledStateOfExecutableButtons();
    }

    protected override void OnClosed(FormClosedEventArgs e) {
      if ((Content != null) && (Content.ExecutionState == ExecutionState.Started)) Content.Stop();
      base.OnClosed(e);
    }

    private void Content_ExperimentChanged(object sender, EventArgs e) {
      experimentNamedItemView.Content = Content.Experiment;
      if (Content.Experiment != null) {
        Content.Prepare();
      }
      SetEnabledStateOfControls();
    }

    #region Content Events
    private void Content_ExecutionStateChanged(object sender, EventArgs e) {
      if (InvokeRequired)
        Invoke(new EventHandler(Content_ExecutionStateChanged), sender, e);
      else
        startButton.Enabled = pauseButton.Enabled = stopButton.Enabled = resetButton.Enabled = false;
    }
    private void Content_Prepared(object sender, EventArgs e) {
      if (InvokeRequired)
        Invoke(new EventHandler(Content_Prepared), sender, e);
      else {
        nameTextBox.Enabled = descriptionTextBox.Enabled = true;
        Locked = false;
        SetEnabledStateOfExecutableButtons();
      }
    }
    private void Content_Started(object sender, EventArgs e) {
      if (InvokeRequired)
        Invoke(new EventHandler(Content_Started), sender, e);
      else {
        nameTextBox.Enabled = descriptionTextBox.Enabled = false;
        Locked = true;
        SetEnabledStateOfExecutableButtons();
      }
    }
    private void Content_Paused(object sender, EventArgs e) {
      if (InvokeRequired)
        Invoke(new EventHandler(Content_Paused), sender, e);
      else {
        nameTextBox.Enabled = descriptionTextBox.Enabled = true;
        Locked = false;
        SetEnabledStateOfExecutableButtons();
      }
    }
    private void Content_Stopped(object sender, EventArgs e) {
      if (InvokeRequired)
        Invoke(new EventHandler(Content_Stopped), sender, e);
      else {
        nameTextBox.Enabled = descriptionTextBox.Enabled = true;
        Locked = false;
        SetEnabledStateOfExecutableButtons();
      }
    }
    private void Content_ExecutionTimeChanged(object sender, EventArgs e) {
      if (InvokeRequired)
        Invoke(new EventHandler(Content_ExecutionTimeChanged), sender, e);
      else
        executionTimeTextBox.Text = Content.ExecutionTime.ToString();
    }
    private void Content_ExceptionOccurred(object sender, EventArgs<Exception> e) {
      if (InvokeRequired)
        Invoke(new EventHandler<EventArgs<Exception>>(Content_ExceptionOccurred), sender, e);
      else
        ErrorHandling.ShowErrorDialog(this, e.Value);
    }
    #endregion

    #region Control events
    private void startButton_Click(object sender, EventArgs e) {
      Content.Start();
    }
    private void pauseButton_Click(object sender, EventArgs e) {
      Content.Pause();
    }
    private void stopButton_Click(object sender, EventArgs e) {
      Content.Stop();
    }
    private void resetButton_Click(object sender, EventArgs e) {
      Content.Prepare();
    }
    #endregion

    #region Helpers
    private void SetEnabledStateOfExecutableButtons() {
      if (Content == null) {
        startButton.Enabled = pauseButton.Enabled = stopButton.Enabled = resetButton.Enabled = false;
      } else {
        startButton.Enabled = (Content.ExecutionState == ExecutionState.Prepared) || (Content.ExecutionState == ExecutionState.Paused);
        //pauseButton.Enabled = Content.ExecutionState == ExecutionState.Started;
        pauseButton.Enabled = false; // disabled for now
        stopButton.Enabled = (Content.ExecutionState == ExecutionState.Started) || (Content.ExecutionState == ExecutionState.Paused);
        resetButton.Enabled = Content.ExecutionState != ExecutionState.Started;
        
      }
    }
    #endregion

    private void serverUrlTextBox_Validating(object sender, System.ComponentModel.CancelEventArgs e) {
      if (string.IsNullOrEmpty(serverUrlTextBox.Text)) {
        e.Cancel = true;
        errorProvider.SetError(serverUrlTextBox, "ServerUrl cannot be empty");
        serverUrlTextBox.SelectAll();
        return;
      }
      Content.ServerUrl = serverUrlTextBox.Text;
    }

    private void serverUrlTextBox_Validated(object sender, EventArgs e) {
      errorProvider.SetError(serverUrlTextBox, string.Empty);
    }

    private void resourceIdsTextBox_Validated(object sender, EventArgs e) {
      Content.ResourceIds = resourceIdsTextBox.Text;
    }

    private void newExperimentButton_Click(object sender, EventArgs e) {
      Content.Experiment = new HeuristicLab.Optimization.Experiment();
    }

    private void loadExperimentButton_Click(object sender, EventArgs e) {
      OpenFileDialog openFileDialog = new OpenFileDialog();
      openFileDialog.Title = "Open Experimenter";
      openFileDialog.FileName = "Item";
      openFileDialog.Multiselect = false;
      openFileDialog.DefaultExt = "hl";
      openFileDialog.Filter = "HeuristicLab Files|*.hl|All Files|*.*";

      if (openFileDialog.ShowDialog() == DialogResult.OK) {
        Content.Experiment = (HeuristicLab.Optimization.Experiment)ContentManager.Load(openFileDialog.FileName);
      }
    }

    private void openExperimentButton_Click(object sender, EventArgs e) {
      MainFormManager.MainForm.ShowContent(Content.Experiment);
    }

  }
}
